/*
 * print-part.c
 *
 * 
 * Author: 
 *  Richard Hult <rhult@hem.passagen.se>
 * 
 *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
 * 
 * Copyright (C) 1999,2000  Richard Hult 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the 
 * License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <math.h>
#include <gnome.h>
#include "main.h"
#include "sheet-private.h"
#include "item-data.h"
#include "part.h"
#include "part-label.h"
#include "part-property.h"
#include "load-library.h"
#include "part-private.h"
#include "print.h"
#include "print-part.h"

//static void draw_ellipse (GnomePrintContext *ctx, ArtPoint *center, gdouble width, gdouble height);
//static void draw_text (GnomePrintContext *ctx, const gchar *text, ArtPoint *pos);

void
print_part (OreganoPrintContext *opc, Part *part)
{
#if 0
	GSList *objects, *labels;
	SymbolObject *object;
	LibrarySymbol *symbol;
	GnomeCanvasPoints *line;
	double x0, y0;
	ArtPoint dst, src;
	double affine[6];
	int i, rotation;
	PartPriv *priv;
	SheetPos pos;

	g_return_if_fail (part != NULL);
	g_return_if_fail (IS_PART (part));

	priv = part->priv;

	item_data_get_pos (ITEM_DATA (part), &pos);
	x0 = pos.x;
	y0 = pos.y;

	rotation = part_get_rotation (part);
	if (rotation != 0){
		art_affine_rotate (affine, 90 * rotation);
	}

	symbol = library_get_symbol (priv->symbol_name);
	if (symbol == NULL) {
		return;
	}

	for (objects = symbol->symbol_objects; objects; objects = objects->next){
		object = (SymbolObject *)(objects->data);

		switch (object->type)
		{
		case SYMBOL_OBJECT_LINE:
			gnome_print_setlinewidth (opc->ctx, 0);
			line = object->u.line;
			for (i = 0; i < line->num_points; i++){
				double x, y;

				x = line->coords[i * 2];
				y = line->coords[i * 2 + 1];

				if (rotation != 0){
					src.x = x;
					src.y = y;
					art_affine_point (&dst, &src, affine);
					x = dst.x;
					y = dst.y;
				}

				if (i == 0)
					gnome_print_moveto (opc->ctx, x0 + x, y0 + y);
				else
					gnome_print_lineto (opc->ctx, x0 + x, y0 + y);
			}
			break;

		case SYMBOL_OBJECT_ARC:
		{
			gdouble x1 = object->u.arc.x1;
			gdouble y1 = object->u.arc.y1;
			gdouble x2 = object->u.arc.x2;
			gdouble y2 = object->u.arc.y2;
			gdouble width, height;
			ArtPoint center;

			center.x = (x2 - x1) / 2 + x1 + x0;
			center.y = (y2 - y1) / 2 + y1 + y0;

			width = x2 - x1; 
			height = y2 - y1; 

			gnome_print_setlinewidth (opc->ctx, 0);
			draw_ellipse (opc->ctx, &center, width, height);
			break;
		}
		default:
			g_warning ("Print part: Part %s contains unknown object.", priv->name);
			continue;
		}

		gnome_print_stroke (opc->ctx);

		for (labels = part_get_labels (part); labels; labels = labels->next) {
			PartLabel *label = labels->data;
			ArtPoint label_pos;
			gchar *text;
			
			label_pos.x = label->pos.x;
			label_pos.y = label->pos.y;

			if (rotation != 0){
				src.x = label_pos.x;
				src.y = label_pos.y;
				art_affine_point (&dst, &src, affine);
				label_pos.x = dst.x;
				label_pos.y = dst.y;
			}

			label_pos.x += x0;
			label_pos.y += y0;

			text = part_property_expand_macros (part, label->text);

			gnome_print_setfont (opc->ctx, opc->label_font);			
			draw_text (opc->ctx, text, &label_pos);
			g_free (text);
		}
	}
#endif
}

#if 0
/*
 * This is stolen from Dia:
 */
#define ELLIPSE_CTRL1 0.26521648984
#define ELLIPSE_CTRL2 0.519570402739
#define ELLIPSE_CTRL3 M_SQRT1_2
#define ELLIPSE_CTRL4 0.894643159635

static void
draw_ellipse(GnomePrintContext *ctx, ArtPoint *center, gdouble width, gdouble height)
{
	gdouble x1 = center->x - width/2,  x2 = center->x + width/2;
	gdouble y1 = center->y - height/2, y2 = center->y + height/2;
	gdouble cw1 = ELLIPSE_CTRL1 * width / 2, cw2 = ELLIPSE_CTRL2 * width / 2;
	gdouble cw3 = ELLIPSE_CTRL3 * width / 2, cw4 = ELLIPSE_CTRL4 * width / 2;
	gdouble ch1 = ELLIPSE_CTRL1 * height / 2, ch2 = ELLIPSE_CTRL2 * height / 2;
	gdouble ch3 = ELLIPSE_CTRL3 * height / 2, ch4 = ELLIPSE_CTRL4 * height / 2;

	gnome_print_moveto (ctx, x1, center->y);
	gnome_print_curveto (ctx,
			    x1,              center->y - ch1,
			    center->x - cw4, center->y - ch2,
			    center->x - cw3, center->y - ch3);
	gnome_print_curveto (ctx,
			    center->x - cw2, center->y - ch4,
			    center->x - cw1, y1,
			    center->x,       y1);
	gnome_print_curveto (ctx,
			    center->x + cw1, y1,
			    center->x + cw2, center->y - ch4,
			    center->x + cw3, center->y - ch3);
	gnome_print_curveto (ctx,
			    center->x + cw4, center->y - ch2,
			    x2,              center->y - ch1,
			    x2,              center->y);
	gnome_print_curveto (ctx,
			    x2,              center->y + ch1,
			    center->x + cw4, center->y + ch2,
			    center->x + cw3, center->y + ch3);
	gnome_print_curveto (ctx,
			    center->x + cw2, center->y + ch4,
			    center->x + cw1, y2,
			    center->x,       y2);
	gnome_print_curveto (ctx,
			    center->x - cw1, y2,
			    center->x - cw2, center->y + ch4,
			    center->x - cw3, center->y + ch3);
	gnome_print_curveto (ctx,
			    center->x - cw4, center->y + ch2,
			    x1,              center->y + ch1,
			    x1,              center->y);
}

static void
draw_text (GnomePrintContext *ctx, const gchar *text, ArtPoint *pos)
{
	gnome_print_moveto (ctx, pos->x, pos->y);
	
	gnome_print_gsave (ctx);
	gnome_print_scale (ctx, 1, -1);
	gnome_print_show (ctx, text);
	gnome_print_grestore (ctx);
}
#endif
