/*
 * sheet-node.c
 *
 * 
 * Author: 
 *  Richard Hult <rhult@hem.passagen.se>
 * 
 *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
 * 
 * Copyright (C) 1999,2000  Richard Hult 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the 
 * License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <gnome.h>
#include "sheet-node.h"
#include "part.h"

static void sheet_node_class_init (SheetNodeClass *klass);
static void sheet_node_init (SheetNode *node);

enum {
	SOMETHING,
	LAST_SIGNAL
};
/*static guint sheet_node_signals [LAST_SIGNAL] = { 0 };*/
static GtkObject *parent_class = NULL;

guint
sheet_node_get_type (void)
{
	static guint sheet_node_type = 0;

	if (!sheet_node_type) {
		static const GtkTypeInfo sheet_node_info = {
			"SheetNode",
			sizeof (SheetNode),
			sizeof (SheetNodeClass),
			(GtkClassInitFunc) sheet_node_class_init,
			(GtkObjectInitFunc) sheet_node_init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL,
		};
		
		sheet_node_type = gtk_type_unique (GTK_TYPE_OBJECT, &sheet_node_info);
	}

	return sheet_node_type;
}

static void
sheet_node_destroy (GtkObject *object)
{
/*	SheetNode *node = SHEET_NODE (object);*/

	GTK_OBJECT_CLASS (parent_class)->destroy (object);
}

static void
sheet_node_class_init (SheetNodeClass *klass)
{
	GtkObjectClass *object_class;

	object_class = (GtkObjectClass *)klass;
	parent_class = gtk_type_class (GTK_TYPE_OBJECT);

	object_class->destroy = sheet_node_destroy;

/*	sheet_node_signals [VOLTAGE_CHANGED] = 
		gtk_signal_new ("something",
				GTK_RUN_FIRST,
				object_class->type,
				GTK_SIGNAL_OFFSET (sheetNodeClass, something),
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);

	gtk_object_class_add_signals (object_class, sheet_node_signals, LAST_SIGNAL);
*/
}

static void
sheet_node_init (SheetNode *node)
{
	node->pin_count = 0;
	node->wire_count = 0;
	node->pins = NULL;
	node->wires = NULL;
	node->visited = FALSE;
}

SheetNode *
sheet_node_new (SheetPos pos)
{
	SheetNode *node;

	node = gtk_type_new (sheet_node_get_type ());

	node->key = pos;

	return node;
}

gint
sheet_node_add_pin (SheetNode *node, Pin *pin)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);
	g_return_val_if_fail (pin != NULL, FALSE);

	if (g_slist_find (node->pins, pin)) {
/*		g_print ("sheet_node_add_pin: pin already there.\n");*/
		return FALSE;
	}
	
	node->pins = g_slist_prepend (node->pins, pin);
	node->pin_count++;

	return TRUE;
}

gint
sheet_node_remove_pin (SheetNode *node, Pin *pin)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);
	g_return_val_if_fail (pin != NULL, FALSE);

	if (node->pin_count == 0)
		return FALSE;

	node->pins = g_slist_remove (node->pins, pin);
	node->pin_count--;

	return TRUE;
}

gint
sheet_node_add_wire (SheetNode *node, Wire *wire)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);
	g_return_val_if_fail (wire != NULL, FALSE);
	g_return_val_if_fail (IS_WIRE (wire), FALSE);

	if (g_slist_find (node->wires, wire)) {
/*		g_print ("sheet_node_add_wire: wire already there.\n");*/
		return FALSE;
	}

	node->wires = g_slist_prepend (node->wires, wire);
	node->wire_count++;

	return TRUE;
}

gint
sheet_node_remove_wire (SheetNode *node, Wire *wire)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);
	g_return_val_if_fail (wire != NULL, FALSE);
	g_return_val_if_fail (IS_WIRE (wire), FALSE);

	if (node->wire_count == 0)
		return FALSE;

	if (!g_slist_find (node->wires, wire)) {
		g_print ("sheet_node_remove_wire: not there.\n");
		return FALSE;
	}

	node->wires = g_slist_remove (node->wires, wire);
	node->wire_count--;

	return TRUE;
}

gint
sheet_node_is_empty (SheetNode *node)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);

	if (node->wire_count == 0 && node->pin_count == 0)
	    return TRUE;

	return FALSE;
}

gint
sheet_node_is_visited (SheetNode *node)
{
	g_return_val_if_fail (node != NULL, FALSE);
	g_return_val_if_fail (IS_SHEET_NODE (node), FALSE);
	
	return node->visited;
}

void
sheet_node_set_visited (SheetNode *node, gboolean is_visited)
{
	g_return_if_fail (node != NULL);
	g_return_if_fail (IS_SHEET_NODE (node));
	
	node->visited = is_visited;
}
