/*
 * sim-engine.h
 *
 * 
 * Author: 
 *  Richard Hult <rhult@hem.passagen.se>
 * 
 *  http://www.dtek.chalmers.se/~d4hult/oregano/ 
 * 
 * Copyright (C) 1999,2000  Richard Hult 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the 
 * License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __SIM_ENGINE_H
#define __SIM_ENGINE_H

#include <gtk/gtk.h>
#include "sim-settings.h"

#define TYPE_SIM_ENGINE            (sim_engine_get_type ())
#define SIM_ENGINE(obj)            (GTK_CHECK_CAST ((obj), TYPE_SIM_ENGINE, SimEngine))
#define SIM_ENGINE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_SIM_ENGINE, SimEngineClass))
#define IS_SIM_ENGINE(obj)         (GTK_CHECK_TYPE ((obj), TYPE_SIM_ENGINE))
#define IS_SIM_ENGINE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_SIM_ENGINE))

typedef struct _SimEngine SimEngine;
typedef struct _SimEngineClass SimEngineClass;

#include "schematic.h"

typedef struct {
	int    state;

	char **var_names;
	char **var_units;
	int    num_var;
	int    got_var;

	GArray **data;
	int      num_data;

	int    got_points;
} SimOp;

typedef struct {
	double  x;
} SimFourier;

typedef struct {
	gint    state;

	gdouble sim_length;
	gdouble step_size;

	gchar **var_names;
	gchar **var_units;
	gint    num_var;
	gint    got_var;

	GArray **data;
	gint      num_data;
	gdouble  *min_data;
	gdouble  *max_data;

	gint    got_points;
} SimTransient;

struct _SimEngine {
	GtkObject parent;

	Schematic   *sm;

	SimOp        op;
	SimTransient transient;
	SimFourier   fourier;

	SimSettings *sim_settings;
	
	gboolean     running;
	gdouble      progress;
	
	pid_t        child_pid;
	gint         data_input_id;
	gint         error_input_id;
	gint         to_child[2];
	gint         to_parent[2];
	gint         to_parent_error[2];
	FILE        *inputfp;
	FILE        *outputfp;
	FILE        *errorfp;

	gboolean     has_warnings;
	gboolean     has_errors;
};

struct _SimEngineClass
{
	GtkObjectClass parent_class;
	
	void (*done) (SimEngine *engine);
	void (*aborted) (SimEngine *engine);
	void (*cancelled) (SimEngine *engine);
};

guint sim_engine_get_type (void);
SimEngine *sim_engine_new (Schematic *sm);
void sim_engine_start_with_file (SimEngine *engine, const gchar *netlist);
void sim_engine_stop (SimEngine *engine);


#endif
